# -*- coding: utf-8 -*-
#
# Author: Ludwig Reiter
from datetime import date
import logging
import os.path
import tempfile
import re

from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.db import connection
from django.http import HttpResponse
from django.shortcuts import render

import openpyxl

logger = logging.getLogger(__name__)


@login_required
def overview(request):
    return render(request, "fesreport.html", {
        "foerderphasen": settings.FOERDERPHASEN,
    })


@login_required
def programm_download(request):
    return export_xlsx('Programmebene',
                       'FRUEHERKENNUNG_TEMPLATE')


@login_required
def beneficiary_download(request):
    foerderphase = request.GET.get("fp")
    if foerderphase is not None:
        foerderphase = int(foerderphase)
    return export_xlsx('Vorhabenebene',
                       'FRUEHERKENNUNG_VORHABEN_TEMPLATE', foerderphase)


# Process template
def fill_sheet(ws, foerderphase):
    """Process the given worksheet.

    Replace all cell values
        fes:[indicator]:[gender]:[regioncategory]
    or:
        fes:[indicator]:[gender]:[regioncategory]:[beneficiary]
    - indicator:      po04, a2_1, etc., also: a2_1+exit
                      Special Keys: entry, exit, target
    - gender:         m, f, t (male, female, total)
    - regioncategory: 1, 2+4, ...
    - beneficiary (optional): Beneficiary ID (eg. PWE.0123.14)
    """

    FES = re.compile(
        r'fes:(?P<indicator>[^:]+):(?P<gender>[^:]+)'
        r':(?P<regioncategory>[^:]+)(:(?P<beneficiary>[^:]+))?')
    for row in ws.rows:
        for cell in row:
            fes = FES.search(u"{}".format(cell.value))  # convert to string
            if fes:
                # indicator
                indicator = fes.group('indicator').lower()
                conditions = []
                for i in indicator.split('+'):
                    # rewrite the where condition
                    if i == 'exit':
                        conditions.append('completeness_exit = 100')
                    elif indicator == 'entry':
                        conditions.append('true')  # all entries are countable
                    else:
                        conditions.append(i)
                indicator_condition = ' AND '.join(conditions)

                # gender
                gender = fes.group('gender')[0].lower()
                # rewrite the where condition
                if gender == 'm':
                    gender = "= 1"  # male
                elif gender == 'f':
                    gender = "= 0"  # female
                else:
                    gender = 'in (0,1)'  # total

                # regioncategory
                if fes.group('regioncategory') == '*':
                    rc_condition = "TRUE"
                else:
                    rc = ','.join(fes.group('regioncategory').split('+'))
                    rc_condition = 'regioncategory_id in ({})'.format(rc)

                # beneficiary
                # Strip the last 3 chars from beneficiary and search
                # for pattern with LIKE
                if fes.group('beneficiary'):
                    beneficiary_condition = (
                        "originating_beneficiary_id LIKE '{}%'".format(
                            fes.group('beneficiary')[:-3]))
                else:
                    beneficiary_condition = "TRUE"

                # SELECT
                if indicator == 'target':
                    phase_condition = "fp='{}'".format(
                        foerderphase) if foerderphase is not None else "TRUE"
                    select = ('SELECT sum(oi_target_number) '
                              ' FROM beneficiary '
                              ' WHERE {}'
                              ' AND {} AND {}').format(rc_condition,
                                                       beneficiary_condition,
                                                       phase_condition)
                elif indicator == 'target_ri_percent':
                    phase_condition = "fp='{}'".format(
                        foerderphase) if foerderphase is not None else "TRUE"
                    select = ("""SELECT avg(ri_target_percent)/10000.0
                               FROM beneficiary 
                               WHERE {}
                               AND {} AND {}""").format(rc_condition,
                                                       beneficiary_condition,
                                                       phase_condition)
                else:
                    if foerderphase is not None:
                        entrydate = "start_date < '2019-01-01'" if foerderphase == 1 else "start_date > '2018-12-31'"
                    else:
                        entrydate = "TRUE"
                    select = ('SELECT count(*) '
                              ' FROM eval_virtual_participations '
                              ' WHERE {} AND gender {} '
                              'AND {}'
                              ' AND {}'
                              ' AND {}').format(indicator_condition,
                                                gender,
                                                entrydate,
                                                rc_condition,
                                                beneficiary_condition)

                # fetch result
                with connection.cursor() as cursor:
                    cursor.execute(select)
                    cell.value = cursor.fetchone()[0]


# work with the xlsx document.
def calculate_difference_in_months(start_date, end_date):
    """Calculates the difference between to dates
        end_date (date): Defaults to None. upper bound
        start_date (date): Defaults to None. [description]

    Every partial month is counted. 01.01.2017 - 01.01.2018
    is counted as 13 months.

    Returns:
        Integer: months
    """

    months_from_years = (end_date.year - start_date.year) * 12
    return months_from_years + end_date.month - start_date.month + 1


def open_workbook(filename):
    return openpyxl.load_workbook(filename=filename)


def get_sheet(workbook):
    # name of the sheet is template specific.
    return workbook[u"Sicht_Progr"]


def export_xlsx_as_string(template, foerderphase):
    with tempfile.NamedTemporaryFile() as tmpfile:
        workbook = open_workbook(get_template_xlsx(template, foerderphase))
        sheet = get_sheet(workbook)
        fill_sheet(sheet, foerderphase)
        write_metadate(sheet)
        workbook.save(tmpfile.name)
        tmpfile.seek(0)
        xlsx_str = tmpfile.read()
    return xlsx_str


def write_metadate(sheet):
    """Write the current date into the meta data section.

    All other dates and calculations are already in the template.
    """
    today = date.today()

    sheet['B6'].value = today
    sheet['B6'].number_format = 'dd.mm.yyyy'


def export_xlsx(festype, template, foerderphase=None):
    xlsx_str = export_xlsx_as_string(template, foerderphase)
    fp_suffix = ""
    if foerderphase:
        fp_suffix = "_FP{}".format(foerderphase)
    response = HttpResponse(xlsx_str,
                            content_type="application/vnd.openxmlformats-"
                                         "officedocument.spreadsheetml.sheet")
    result_name = "{}_{}_Frueherkennung_{}{}.xlsx".format(
        settings.PROGRAM, festype, str(date.today()), fp_suffix)
    response['Content-Disposition'] = ('attachment; ' +
                                       'filename=' + result_name)
    return response


def get_template_xlsx(template, foerderphase):
    suffix = "_FP{}.xlsx".format(
        foerderphase) if foerderphase == 2 else ".xlsx"
    return os.path.join(settings.STATIC_ROOT, "documents",
                        getattr(settings, template)+suffix)
